<?php
namespace tests\Zeedhi\Framework\Process;

use \Zeedhi\Framework\Cache\Type\MemcachedImpl;
use \Zeedhi\Framework\Process\ProcessImpl;
use \Zeedhi\Framework\Process\ProcessManagerImpl;

class ProcessManagerTest extends \PHPUnit\Framework\TestCase{

    /** @var ProcessManagerImpl $process The process object. */
    private $processManager;
    /** @var string $id The first process id. */
    private $firstId;
    /** @var string $id The second process id. */
    private $secondId;
    /* @var MemcachedImpl $cache The cache object to deal with processes statuses. */
    private $cache;

    protected function setUp() {
        $memcached = new \Memcached();
        $this->cache = new MemcachedImpl($memcached);

        $this->processManager = new ProcessManagerImpl($this->cache);
    }

    public function tearDown(){
        $memcached = $this->cache->getMemcached();
        $memcached->flush();
        $memcached->quit();
    }

    public function testAddProcesses() {
        $callback = function(){};
        $firstProcess = new ProcessImpl($callback);
        $secondProcess = new ProcessImpl($callback);
        $this->assertTrue(
            $this->processManager->addProcess($firstProcess) &&
            $this->processManager->addProcess($secondProcess)
        );
    }

    public function testDiscard(){
        $callback = function(){};
        $process = new ProcessImpl($callback);
        $this->processManager->addProcess($process);
        $id = $process->getId();
        $this->assertTrue($this->processManager->discard($id));
        $this->assertEquals(
            'absent',
            $this->processManager->getProcessStatus($id)
        );
    }

    public function testRun() {
        $callback = function(){};
        $process = new ProcessImpl($callback);
        $this->processManager->addProcess($process);
        $id = $process->getId();
        $this->assertTrue($this->processManager->run($id));
    }

    public function testGetProcessStatus(){
        $callback = function(){};
        $process = new ProcessImpl($callback);
        $this->processManager->addProcess($process);
        $id = $process->getId();
        $this->assertEquals(
            'queued',
            $this->processManager->getProcessStatus($id)
        );
    }

    /**
     * Can't get status of 'started' but not 'finished' processes
     * inside single phpUnit test since PHP is single-threaded.
     *
     * Need to mock a server endpoit and start a process then
     * get it's process status by it's id from the cache.
     *
     * Alternatively, you can use reactPHP and pool packages
     * combining the use of a closure child to make it really
     * work as an async process.
     *
     * Not a valuable enough test for the environment effort.
     * Neither an evolution to take place.
     */
     public function testGetProcessStatusStartedProcess(){}

    public function testGetProcessStatusFinishedProcess(){
        $callback = function(){};
        $process = new ProcessImpl($callback);
        $this->processManager->addProcess($process);
        $id = $process->getId();
        $this->processManager->run($id);
        $this->assertEquals(
            'finished',
            $this->processManager->getProcessStatus($id)
        );
    }

    public function testGetProcessStatusFinishedProcessSecondTime(){
        $callback = function(){};
        $process = new ProcessImpl($callback);
        $this->processManager->addProcess($process);
        $id = $process->getId();
        $this->processManager->run($id);
        $this->processManager->getProcessStatus($id);
        $this->assertEquals(
            'absent',
            $this->processManager->getProcessStatus($id)
        );
    }

    public function testGetProcessStatusOfNonExistentProcess(){
        $this->assertEquals(
            'absent',
            $this->processManager->getProcessStatus('non-existent-id')
        );
    }

    /**
     * This test can't be run since it kills the PHP process.
     * And there's no wrong case test for this, since it will
     * anyways terminate all php processes.
     */
     public function testKillAll(){}
}
